@@ -184,6 +184,8 @@ public:
 		static const char *chkContinue(script *img, script::header *scr, script::line_t *line);
 		static const char *chkBreak(script *img, script::header *scr, script::line_t *line);
 		static const char *chkLoop(script *img, script::header *scr, script::line_t *line);
+		static const char *chkPrevious(script *img, script::header *scr, script::line_t *line);
+		static const char *chkIndex(script *img, script::header *scr, script::line_t *line);
 		static const char *chkFor(script *img, script::header *scr, script::line_t *line);
 		static const char *chkForeach(script *img, script::header *scr, script::line_t *line);
 		static const char *chkCase(script *img, script::header *scr, script::line_t *line);
@@ -350,6 +352,7 @@ public:
 		virtual const char *getTypeinit(const char *type_id);
 		virtual const char *getFormat(script::symbol *sym, const char *id, char *temp);
 		virtual bool getCondition(const char *test, const char *value);
+		const char *getIndex(void);
 
 	private:
 		bool getExpression(unsigned index);
@@ -410,6 +413,9 @@ public:
 		bool scrContinue(void);
 		bool scrFor(void);
 		bool scrForeach(void);
+		bool scrPrevious(void);
+		bool scrRepeat(void);
+		bool scrIndex(void);
 		bool scrCase(void);
 		bool scrEndcase(void);
 		bool scrOtherwise(void);
@@ -462,6 +468,9 @@ public:
 	 */
 	static void init(void);
 
+	static void copy(const char *list, char *item, unsigned size);
+	static unsigned count(const char *list);
+	static const char *get(const char *list, unsigned offset);
 	static header *find(script *img, const char *id);
 	static bool isEvent(header *scr, const char *id);
 
