@@ -144,6 +144,28 @@ const char *script::checks::chkApply(script *img, script::header *scr, script::l
 	return NULL;
 }
 
+const char *script::checks::chkPrevious(script *img, script::header *scr, script::line_t *line)
+{
+	method_t method = img->looping();
+
+	if(method == (method_t)NULL)
+		return "cannot be called outside loop";
+
+	if(method == (method_t)&script::methods::scrFor)
+		goto valid;
+
+	if(method == (method_t)&script::methods::scrForeach)
+		goto valid;
+
+	return "cannot be called outside for or foreach block";
+
+valid:
+	if(line->argc)
+		return "command has no arguments";
+
+	return NULL;
+}
+
 const char *script::checks::chkContinue(script *img, script::header *scr, script::line_t *line)
 {
 	method_t method = img->looping();
@@ -183,13 +205,16 @@ const char *script::checks::chkStrict(script *img, script::header *scr, script::
 	if(scr->first) 
 		return "strict must be defined at start of initialization section";
 
-	if(!line->argc)
-		return "strict must define at least one internal global such as \'error\'";
+	if(!line->argc) {
+		script::strict::createGlobal(img, "error");
+		strict::strict::createGlobal(img, "index");
+		return NULL;
+	}
 
 	while(index < line->argc) {
 		cp = line->argv[index++];
 		if(!isalpha(*cp))
-			return "struct must only declare names of defined internal global vars";
+			return "strict must only declare names of defined internal global vars";
 		script::strict::createGlobal(img, cp);	
 	}
 	return NULL;
@@ -248,7 +273,7 @@ const char *script::checks::chkForeach(script *img, script::header *scr, script:
 	if(!line->argc)
 		return "no symbols to assign";
 
-	if(line->argc != 2)
+	if(line->argc < 2 || line->argc > 3)
 		return "assign from only one source";
 
 	cp = line->argv[0];
@@ -271,6 +296,9 @@ const char *script::checks::chkForeach(script *img, script::header *scr, script:
 	if(!isValue(line->argv[1]))
 		return "cannot assign from label or expression";
 
+	if(line->argc == 3 && !isValue(line->argv[2]))
+		return "skip must be a value or symbol";
+
 	return NULL;
 }
 
@@ -834,6 +862,44 @@ const char *script::checks::chkConditional(script *img, script::header *scr, scr
 	return "conditional expression missing";
 }
 	
+const char *script::checks::chkIndex(script *img, script::header *scr, script::line_t *line)
+{
+	unsigned index = 0;
+	const char *cp;
+	method_t method = img->looping();
+
+	if(method == (method_t)NULL)
+		return "cannot be called outside loop";
+
+	if(method == (method_t)&script::methods::scrFor)
+		goto valid;
+
+	if(method == (method_t)&script::methods::scrForeach)
+		goto valid;
+
+	return "cannot be called outside for or foreach block";
+
+valid:
+	if(!line->argc)
+		return "requires at least position";
+
+	while(index < line->argc) {
+		cp = getArgument(line, &index);
+		if(!isValue(cp))
+			return "malformed expression";
+
+		cp = getArgument(line, &index);
+		if(!cp)
+			break;
+		if(!String::equal(cp, "*") && !String::equal(cp, "/") && !String::equal(cp, "+") && !String::equal(cp, "-"))
+			return "invalid expression used";
+		if(index == line->argc)
+			return "incomplete expression";
+	}
+
+	return NULL;
+}
+
 const char *script::checks::chkExpr(script *img, script::header *scr, script::line_t *line)
 {
 	unsigned index = 0;
