@@ -82,6 +82,9 @@ static bool isend(const char *str)
 	if(!strncmp("# ", str, 2))
 		return true;
 
+	if(!strncmp("#\t", str, 2))
+		return true;
+
 	if(!strncmp("//", str, 2))
 		return true;
 
@@ -334,6 +337,9 @@ void script::init(void)
 		{"until", (method_t)&methods::scrUntil, (check_t)&checks::chkUntil},
 		{"break", (method_t)&methods::scrBreak, (check_t)&checks::chkBreak},
 		{"continue", (method_t)&methods::scrContinue, (check_t)&checks::chkContinue},
+		{"previous", (method_t)&methods::scrPrevious, (check_t)&checks::chkPrevious},
+		{"repeat", (method_t)&methods::scrRepeat, (check_t)&checks::chkPrevious},
+		{"index", (method_t)&methods::scrIndex, (check_t)&checks::chkIndex},
 		{"loop", (method_t)&methods::scrLoop, (check_t)&checks::chkLoop},
 		{"while", (method_t)&methods::scrWhile, (check_t)&checks::chkWhile},
 		{"for", (method_t)&methods::scrFor, (check_t)&checks::chkFor},
@@ -403,6 +409,7 @@ script *script::compile(const char *fn, script *cfg)
 	bool label = false;
 	unsigned pos;
 	const char *cp;
+	bool requires = false;
 
 	if(!fp)
 		return NULL;
@@ -484,6 +491,45 @@ initial:
 		tokens = NULL;
 		token = String::token(buffer.c_mem(), &tokens, " \t", "{}\'\'\"\"");
 
+		if(String::equal(token, "endreq") || String::equal(token, "endrequires")) {
+			if(indented) {
+				img->errlog(lnum, "endreq cannot be indented");
+				continue;
+			}
+			requires = false;
+			continue;
+		}
+		else if(String::equal(token, "requires")) {
+			if(indented) {
+				img->errlog(lnum, "requires cannot be indented");
+				continue;
+			}
+			requires = true;
+			while(NULL != (token = String::token(NULL, &tokens, " \t", "{}\'\'\"\""))) {
+				bool rev = false;
+				if(*token == '!') {
+					rev = true;
+					++token;
+				}
+				keyword = find(token);
+				invoke = find(img, token);
+				if(!invoke && is(img->shared))
+					invoke = find(*(img->shared), token);
+				if(!rev && (keyword || invoke)) {
+					requires = false;
+					break;
+				}
+				if(rev && !keyword && !invoke) {
+					requires = false;
+					break;
+				}
+			}
+			continue;
+		}
+
+		if(requires)
+			continue;
+
 		if(*token == '^') {
 			current = (event *)img->alloc(sizeof(script::event));	
 			if(!prior)
@@ -925,3 +971,85 @@ void script::strict::put(FILE *fp, const char *header)
 		fputc(*(pid++), fp);
 	fputc('\n', fp);
 }
+
+unsigned script::count(const char *data)
+{
+	unsigned count = 0;
+	char quote = 0;
+
+	if(!data || !*data)
+		return 0;
+
+	++count;
+	while(*data) {
+		if(*data == ',' && !quote)
+			++count;
+		else if(*data == quote)
+			quote = 0;
+		else if(*data == '\"')
+			quote = *data;
+		++data;
+	}
+	return count;		
+}
+
+void script::copy(const char *list, char *item, unsigned size)
+{
+	assert(size > 0);
+	assert(item != NULL);
+
+	bool lead = false;
+	char quote = 0;
+	char prev = 0;
+	
+	if(!list || !*list || *list == ',') {
+		*item = 0;
+		return;
+	}
+
+	if(*list == '\"') {
+		quote = *(list++);
+		lead = true;
+	}
+
+	while(--size && *list) {
+		if(*list == ',' && !quote)
+			break;
+		if(*list == quote && lead)
+			break;
+		if(*list == quote) {
+			*(item++) = quote;
+			break;
+		}
+		if(!quote && *list == '\"' && prev == '=')
+			quote = *list;
+		prev = *item;
+		*(item++) = *(list++);
+	}
+	*item = 0;
+}
+
+const char *script::get(const char *list, unsigned index)
+{
+	char quote = 0;
+
+	if(!list || !*list)
+		return NULL;
+
+	if(!index)
+		return list;
+
+	while(*list && index) {	
+		if(*list == ',' && !quote) {
+			--index;
+			if(!index) 
+				return ++list;
+		} else if(*list == quote)
+			quote = 0;
+		else if(*list == '\"')
+			quote = *list;
+		++list;
+	}
+	return NULL;
+}
+
