@@ -97,47 +97,12 @@ char *script::interp::getTemp(void)
 
 const char *script::interp::listOffset(const char *list, unsigned index)
 {
-	char quote = 0;
-
-	if(!list || !*list)
-		return NULL;
-
-	if(!index)
-		return list;
-
-	while(*list && index) {	
-		if(*list == ',' && !quote) {
-			--index;
-			if(!index) 
-				return ++list;
-		} else if(*list == quote)
-			quote = 0;
-		else if(*list == '\"')
-			quote = *list;
-		++list;
-	}
-	return NULL;
+	return script::get(list, index);
 }
 		
 unsigned script::interp::listCount(const char *data)
 {
-	unsigned count = 0;
-	char quote = 0;
-
-	if(!data || !*data)
-		return 0;
-
-	++count;
-	while(*data) {
-		if(*data == ',' && !quote)
-			++count;
-		else if(*data == quote)
-			quote = 0;
-		else if(*data == '\"')
-			quote = *data;
-		++data;
-	}
-	return count;
+	return script::count(data);
 }
 
 const char *script::interp::getFormat(script::symbol *sym, const char *id, char *tp)
@@ -164,7 +129,7 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 		return sym->data;
 	}
 	else if(String::equal(id, "count:", 6))
-		snprintf(tp, script::sizing + 1, "%u", listCount(sym->data)); 
+		snprintf(tp, script::sizing + 1, "%u", script::count(sym->data)); 
 	else if(String::equal(id, "offset/", 7)) {
 		id += 7;
 		pos = atoi(id);
@@ -181,19 +146,8 @@ const char *script::interp::getFormat(script::symbol *sym, const char *id, char
 			if(pos)
 				--pos;
 		}
-		cp = listOffset(sym->data, pos);
-		if(!cp || !*cp)
-			return "";
-		if(*cp == '\"')
-			quote = *(cp++);
-		while(dcount < script::sizing && *cp) {
-			if(*cp == quote)
-				break;
-			if(!quote && *cp == ',')
-				break;
-			tp[dcount++] = *(cp++);
-		}
-		tp[dcount] = 0;
+		cp = script::get(sym->data, pos);
+		script::copy(cp, tp, script::sizing);
 	}
 	else if(String::equal(id, "dec:", 4) && sym->size) {
 		val = atoi(sym->data);
@@ -317,6 +271,8 @@ const char *script::interp::getContent(const char *cp)
 			return getFormat(sym, ++cp, tp);
 		}
 	case '%':
+		if(String::equal(cp, "%index"))
+			return getIndex();
 		sym = find(++cp);
 		if(sym)
 			return sym->data;
@@ -348,6 +304,22 @@ const char *script::interp::getKeyword(const char *id)
 	return NULL;
 }
 
+const char *script::interp::getIndex(void)
+{
+	method_t method;
+
+	unsigned pos = frame;
+	while(pos) {
+		method = stack[--pos].line->method;
+		if(method == (method_t)&script::methods::scrFor || method == (method_t)&script::methods::scrForeach) {
+			char *temp = getTemp();
+			snprintf(temp, script::sizing, "%d", stack[pos].index);
+			return temp;
+		}
+	}
+	return "0";	
+}
+
 script::method_t script::interp::getLooping(void)
 {
 	if(!frame)
