@@ -27,13 +27,24 @@ bool script::methods::scrDo(void)
 	return true;
 }
 
+bool script::methods::scrPrevious(void)
+{
+	--frame;
+	if(stack[frame].index < 2) {
+		stack[frame].index = 0;
+		skip();
+		return scrBreak();
+	}
+	stack[frame].index -= 2;
+	return true;
+}
+
 bool script::methods::scrForeach(void)
 {
 	script::line_t *line = stack[frame].line;
 	script::symbol *sym = find(line->argv[0]);
 	const char *cp;
-	unsigned pos = 0, index = stack[frame].index;
-	char quote = 0;
+	unsigned index = stack[frame].index;
 
 	if(!sym)
 		sym = getVar(line->argv[0]);
@@ -44,25 +55,33 @@ bool script::methods::scrForeach(void)
 	if(!sym->size)
 		return error("symbol not writable");
 
-	cp = listOffset(getContent(line->argv[1]), index);
+	if(!index) {
+		cp = line->argv[2];
+		if(cp && (*cp == '%' || *cp == '&')) {
+			script::symbol *skip = find(cp);
+			if(!skip)
+				skip = getVar(cp);
+			if(skip) {
+				index = atoi(skip->data);
+				if(skip->size)
+					String::set(skip->data, 2, "0");
+			}
+		}
+		else if(NULL != (cp = getContent(cp))) {
+			cp = getContent(cp);
+			if(cp)
+				index = atoi(cp);
+		}
+	}
+
+	cp = script::get(getContent(line->argv[1]), index);
 	if(cp == NULL) {
 		stack[frame].index = 0;
 		skip();
 		return scrBreak();
 	}
 
-	if(*cp == '\"')
-		quote = *(cp++);
-
-	while(pos < sym->size && *cp) {
-		if(*cp == ',' && !quote)
-			break;
-		if(*cp == quote)
-			break;
-		sym->data[pos++] = *(cp++);
-	}
-
-	sym->data[pos] = 0;
+	script::copy(cp, sym->data, sym->size + 1);
 	stack[frame].index = ++index;	
 	push();
 	skip();
@@ -286,6 +305,13 @@ bool script::methods::scrBreak(void)
 	return true;
 }
 
+bool script::methods::scrRepeat(void)
+{
+	--frame;
+	--stack[frame].index;
+	return true;
+}
+
 bool script::methods::scrContinue(void)
 {
 	--frame;
@@ -397,9 +423,46 @@ bool script::methods::scrReturn(void)
 	return false;
 }
 
+bool script::methods::scrIndex(void)
+{
+	const char *op;
+	int result = 0, value;
+
+	stack[frame].index = 0;
+	result = atoi(getValue());
+
+	while(NULL != (op = getValue())) {
+		switch(*op) {
+		case '+':
+			result += atoi(getValue());
+			break;
+		case '-':
+			result -= atoi(getValue());
+			break;
+		case '*':
+			result *= atoi(getValue());
+			break;
+		case '/':
+			value = atoi(getValue());
+			if(value == 0)
+				return error("div by zero");
+			result /= value;
+		}
+	}
+
+	--frame;
+	if(!result) {
+		skip();
+		return scrBreak();
+	}
+	else
+		stack[frame].index = --result;
+
+	return true;
+}
+
 bool script::methods::scrExpr(void)
 {
-	unsigned index = 0;
 	unsigned decimals = script::decimals;
 	const char *id, *cp, *op, *aop;
 	float result, value;
@@ -412,6 +475,7 @@ bool script::methods::scrExpr(void)
 
 	id = getContent();
 
+	stack[frame].index = 0;
 	aop = getValue();
 	result = atof(getValue());
 
